# CPB with a metamodel via PCE & PCA

Zenodo link: https://doi.org/10.5281/zenodo.8377205

Cemented Paste Backfill. Python material for the article:

"A metamodel for confined yield stress flows and parameters’ estimation".
Clément Berger, David Coulette, Paul Vigneaux. Submitted October 2023

- Original PDE model: lubrication approximation with Herschel-Bulkley
- A metamodel is built from the original with PCE & PCA
- Two parameters: B ("rheology") and S ("slope")
- The code can perform an inverse problem with the metamodel to find (B,S), assuming one gives a height profile.

## 1. python dependencies

- os
- zipfile
- time
- numpy
- scipy
- matplotlib
- numpoly
- sklearn
- chaospy

## 2. setting directory

It is required to set the directory of this code. The simplest way is to add the environment variable ZENODO_REPO_CPB to your .bashrc

Another solution is to manually integrate the path of this directory in each file of current repo. To do so, replace /PATH/TO/DIR/ by the path of your directory in the following command in a terminal:
```
cd /PATH/TO/DIR/
find . -type f -exec sed -i -e "s~os.environ\[\"ZENODO_REPO_CPB\"\]~\"/PATH/TO/DIR/\"~g" {} \;
```
**IN BOTH CASES**, be sure to use an absolute path and to end it with a "/".

## 3. usage for inversion (= parameters' estimation)

§ To perform an estimation of B,S from a file (containing the height of the material discretized with 301 points), with a superimposed plot of the fitted profile, use "i.comparison_exp_inversion" as demonstrated in launcher.py

§§ The profile should be uploaded as a text file in the folder data/experiments/, readable by "np.loadtxt(FILENAME)" (i.e. a line corresponds to a point of the profile, for a total of 301 lines). 

§§ If the profile comes from experimental data and has dimensions, a file named "FILENAME_dimensions" has to be placed in the same folder. See the example file "data/experiments/illustrative_profile_dimensions".

§§ If the profile is already non-dimensional, then the parameter "do_nondim" must be set to False in "i.comparison_exp_inversion" and the extra file "FILENAME_dimensions" is not required anymore.

§ For more custom usage, "i.inverse_batch(profiles)" takes a numpy array of size (num_profiles,301) and perform successive inversions for all profiles (one can simply use "i.inverse(profile)" if there is only one profile).

§§ Note that this function can take arguments both for the metamodel or for the Nelder-Mead algorithm, see documentation within the code for details.


## 4. other functionalities ...

... such as training a PCE on another dataset, using the PDE solver and so on are left possible. One should read the documentation within the source code to do so. It is advised to carefully read the documentation of "src/utils/data_base.py" before using/creating new datasets.
